/*
 * This file is a part of Winix
 * and is not publicly distributed
 * 
 * Copyright (c) 2008-2010, Tomasz Sowa
 * All rights reserved.
 * 
 */
 
#include <arpa/inet.h>
#include "users.h"



Users::Users()
{
	how_many_logged = 0;
	Clear();
}


void Users::SetCur(Cur * pcur)
{
	cur = pcur;
}


void Users::Clear()
{
	table.Clear();	
}



void Users::ReadUsers(Db * db)
{
	Clear();	
	db->GetUsers(table);
}


void Users::SetTimeZoneOffset(int offset)
{
	Table::Iterator i;

	for(i=table.Begin() ; i!=table.End() ; ++i)
	{
		(*i)->time_zone_offset = offset;
	}
}


bool Users::AddUser(const User & user)
{
	Table::Iterator i = table.PushBack(user);
	
return (i != table.End());
}


bool Users::IsUser(const std::wstring & name)
{
	return table.Is(name);
}


User * Users::GetUser(long user_id)
{
	Table::Iterator i = table.FindId(user_id);
	
	if( i == table.End() )
		return 0;
	
return &(**i);
}


User * Users::GetUser(const std::wstring & name)
{
	Table::Iterator i = table.FindName(name);
	
	if( i == table.End() )
		return 0;
	
return &(**i);
}



long Users::GetUserId(const std::wstring & name)
{
User * puser = GetUser(name);

	if( !puser )
		return -1;	
	
return puser->id;
}



Users::Iterator Users::Begin()
{
	return table.Begin();
}



Users::Iterator Users::End()
{
	return table.End();
}



Users::SizeType Users::Size()
{
	return table.Size();
}



User & Users::operator[](Users::SizeType pos)
{
	// !! IMPROVE ME
	// add checking the range and returning an empty User struct if out of range
	return table[pos];
}


bool Users::LoginUser(long user_id, bool remember_me, bool use_ses_log)
{
	if( !cur->session )
		return false;

	if( cur->session->id == 0 )
	{
		log << log1 << "Users: cannot login a user on a temporary session" << logend;

		if( use_ses_log )
			slog << logerror << T(L"service_unavailable") << logend;

		return false;
	}

	User * puser = GetUser(user_id);

	if( !puser )
	{
		log << log1 << "Users: user id: " << user_id << " is not in system.users table" << logend;

		if( use_ses_log )
			slog << logerror << T(L"service_unavailable") << logend;

		return false;
	}

	if( puser->status != WINIX_ACCOUNT_READY )
	{
		log << log1 << "Users: user id: " << user_id << " is not ready for logging in (status: "
			<< puser->status << ")" << logend;

		if( use_ses_log )
		{
			if( puser->status == WINIX_ACCOUNT_NOT_ACTIVATED )
				slog << logerror << T(L"account_not_activated") << logend;
			else
			if( puser->status == WINIX_ACCOUNT_SUSPENDED )
				slog << logerror << T(L"account_suspended") << logend;
			else
			if( puser->status == WINIX_ACCOUNT_BLOCKED )
				slog << logerror << T(L"account_banned") << logend;
		}

		return false;
	}

	if( cur->session->puser )
		LogoutCurrentUser();

	cur->session->puser       = puser;
	cur->session->spam_score  = 0;
	cur->session->remember_me = remember_me;

	last.UserLogin(user_id, cur->session->puser->name, inet_addr(cur->request->env_remote_addr), cur->session->id);

	// !! jesli uzytkownik ponownie sie loguje ten sam na ta sama sesje to mozna nie zwiekszac licznika
	how_many_logged += 1;

	log << log2 << "User " << cur->session->puser->name << " (id: " << user_id << ") logged" << logend;

return true;
}




void Users::LogoutCurrentUser()
{
	if( !cur->session || !cur->session->puser )
		return;

	log << log2 << "Users: user " << cur->session->puser->name <<  ", id: "
		<< cur->session->puser->id << " logged out" << logend;
	
	last.UserLogout(cur->session->puser->id, cur->session->id);

	if( how_many_logged > 0 ) // for safety
		how_many_logged -= 1;
	
	cur->session->puser = 0;
	cur->session->remember_me = 0;
}


void Users::IncrementLoggedUsers()
{
	how_many_logged += 1;
}


long Users::HowManyLogged()
{
	return how_many_logged;
}




